% iblfix.pl
% Access functions for .fix-files generated from IBL lexicon files.
% project: IBL
% Sven Hartrumpf (C) 1994


% The following program will already be included by 'german.pre'.

%:- ensure_loaded('ibl.pl').


% iBl_fix_lookup/2
% iBl_fix_lookup(F, L) is true iff
%   L is the list of feature structures that satisfy the equation: form = F.
% iBl_fix_lookup(+character_code_list,?list)

iBl_fix_lookup(Form, L) :-
  iBl_fix_class_lookup(Form, Classes),
  iBl_fix_lookup2(Classes, Form, L).


iBl_fix_lookup2([], _, []).

iBl_fix_lookup2([Class|Classes], Form, L) :-
  iBl_general_extension(Class, L0, [form = Form]),
  iBl_remove_local_part(L0, L1),
  iBl_fix_lookup2(Classes, Form, L2),
  append(L1, L2, L).
  

% iBl_fix_class_lookup/2
% iBl_fix_class_lookup(F, L) is true iff
%   L is the list of the numerical identifiers of the lexical classes that
%   contain at least one feature structure that satisfies the equation: form = F.
% iBl_fix_class_lookup(+character_code_list,?list)

iBl_fix_class_lookup(Form, Classes) :-
  ltree(Tree),
  ltree_find(Form, Tree, Classes).


% ltree_find/3
% ltree_find(F,T,R) is true iff
%   R is the list of class identifiers associated with the word form F in the
%   letter tree T.
% ltree_find(+character_code_list,+term,?list)

ltree_find([], Tree, Entry) :-
  !,
  member(E, Tree),
  (
    integer(E)
  ->
    Entry = [E]
  ;
    E = m(Entry)
  ).

ltree_find(Form, Tree, Entry) :-
  first_letter(Form, First, Rest0),
  member([First|Branches], Tree),
  (
    Branches = [c(S)|_]
  ->
    atom_codes(S,S1),
    append(S1, Rest, Rest0)
  ;
    Rest = Rest0
  ), 
  ltree_find(Rest, Branches, Entry).


% first_letter/3
% first_letter(W,F,R) is true iff
%   - F is the atom corresponding to the first letter of the word W, and
%   - R is the list W without the first element.
% first_letter(+character_code_list,?atom,?character_code_list)

first_letter([F|Rest], First, Rest) :-
  atom_codes(First,[F]).
