// ltree.c
// methods to write a letter tree as a Prolog term
// Sven Hartrumpf (C) 1994


#include "vindex.h"


#include <ctype.h>
#include <iomanip.h>
#include <std.h>


static void write_spaces(ostream & out, UWORD n)
{
  while (n--)
    out << ' ';
}


ULONG
strcmpl(const UBYTE * a, const UBYTE * b, ULONG max)
{
  ULONG i = 0;
  while ((i < max) && (*a++ == *b++))
    i++;
  return i;
}


UBYTE
Vindex<Index_word>::write_ltree() const
{
  UWORD l = strlen((const char*) name_) - 3;
  UBYTE s[l];
  strncpy((char*) s, (const char*) name_, l - 1);
  s[l - 1] = 0;
  cout << "% This is a letter tree file automatically generated from an IBL-file.\n\n";
  cout << ":- ensure_loaded('" << s << ".pre').\n";
  cout << ":- ensure_loaded('iblfix.pl').\n\n";
  cout << "ltree([\n";
  write_ltree_aux(0, 0, num_entries - 1, 0);
  cout << "\n]).\n";
  return TRUE;
}


void
Vindex<Index_word>::write_ltree_aux(UWORD position, ULONG left, ULONG right,
                                    UWORD level) const
{
  ULONG i, j, k;
  const UBYTE * c;
  WORD l, max, max1;
  BYTE first = TRUE;
  i = left;
  while (i <= right)
  {
    c = entry[i].string() + position;
    max = entry[i].length() - position;
    for (j = i + 1; j <= right; j++)
    {
      l = entry[j].length() - position;
      if (l <= 0)
        break;
      max1 = min(max, l); 
      k = strcmpl(entry[j].string() + position, c, max1);
      if (!k)
        break;
      if (k < max)
        max = k;
    }
    if (first)
      first = FALSE;
    else
      cout << ",\n";
    write_branch(position, i, j - 1, max, level);
    i = j;
  }
}


void
Vindex<Index_word>::write_branch(UWORD position, ULONG left, ULONG right,
                                 UWORD l, UWORD level) const
{
  const UWORD max_ids = 20;
  ULONG h, i = left, j, k, n;
  ULONG id[max_ids];
  const UBYTE * s;
  write_spaces(cout, level);
  cout << "[";
  s = entry[left].string() + position;
  if (l > 1)
  { // more than one character at one step
    if (isupper(*s))
    {
      cout << "'";
      cout.put(*s);
      cout << "'";
    }
    else
      cout.put(*s);
    cout << ",c(";
    cout.write(s + 1, l - 1);
    cout << ")";
  }
  else
  {
    if (isupper(*s))
    {
      cout << "'";
      cout.put(*s);
      cout << "'";
    }
    else
      cout.put(*s);
  }
  if (position + l == entry[left].length()) // whole word covered
  {
    cout << ",";
    n = 1;
    id[0] = entry[i].id();
    for (j = i + 1; j <= right; j++)
    {
      if (position + l != entry[j].length())
        break;
      h = entry[j].id();
      for (k = 0; k < n; k++)
        if (id[k] == h)
          break;
      if (k >= n)
      {
        if (n >= max_ids)
        {
          cerr << "Error: more than " << n
            << "different references for one entry.\n";
        }
        else
          id[n++] = h;
      }
    }
    if (n == 1)
    {
      cout << id[0];
      i++;
    }
    else
    {
      cout << "m([" << id[0];
      for (k = 1; k < n; k++)
        cout << "," << id[k];
      cout << "])";
      i = j;
    }
  }
  if (i <= right)
  {  // There is at least one word that has the same prefix and is longer.
    cout << ",\n";
    write_ltree_aux(position + l, i, right, level + 1);
    cout << "\n";
    write_spaces(cout, level);
    cout << "]";
  }
  else
    cout << "]";
}
