// sort.h
// quicksort template
// Sven Hartrumpf (C) 1994


#ifndef SORT_H_

#define SORT_H_



#include "types.h"
#include "util.h"


#include <iomanip.h>
#include <iostream.h>


template<class T> class Comparator
{
public:
  BYTE (*compare)(const T & a, const T & b);
};


template<class T> void
quicksort(T * v, ULONG n, Comparator<T> & comp)
{
  // the third argment should be: BYTE (*compare)(const T & a,const T & b)
  // but this is not implemented in gcc 2.5.7
  if (n < 2)
    return;
  BYTE (*compare)(const T & a, const T & b) = comp.compare;
  ULONG *bottom_of_stack, *top_of_stack;
  WORD i, j, left, mid, right;
  ULONG stack_size = ceiling_ld(n) * 2;
  T median;
  //cout << "stack_size: " << stack_size << '\n';
  top_of_stack = bottom_of_stack = new ULONG[stack_size];
  *top_of_stack++ = 0;
  *top_of_stack++ = n-1;
  do
  {
    right = *--top_of_stack;
    left = *--top_of_stack;
    do
    {
      i = left;
      j = right;
      //cout << "quicksort from " << i << " to " << j << '\n';
      /*for (ULONG h = left; h <= right; h++)
      {
        cout << setw(6) << h << " ";
        v[h].print();
        cout << '\n';
      }*/
      mid = (i + j) / 2;
      median = v[mid];
      do
      {
        while (compare(v[i], median) < 0)
          i++;
        while (compare(v[j], median) > 0)
          j--;
        //cout << "after compare i: " << i << " j: " << j << '\n';
        if (i <= j)
        {
          //cout << "quicksort swaps elements " << i << " and " << j << '\n';
          if (i < j)
            swap(v[i], v[j]);
          i++;
          j--;
        }
      }
      while (i <= j);
      //cout << "after partitioning\n";
      /*for (h = left; h <= right; h++)
      {
        cout << setw(6) << h << " ";
        v[h].print();
        cout << '\n';
      }*/
      if (j - left < right - i)
      { // right part is greater
        if (i < right) // push the right part on the stack
        {
          *top_of_stack++=i;
          *top_of_stack++=right;
        }
        right = j; // continue with the left part
      }
      else
      { // left part is greater
        if (left < j) // push the left part on the stack
        {
          *top_of_stack++ = left;
          *top_of_stack++ = j;
        }
        left = i; // continue with the right part
      }
    }
    while (left < right);
  }
  while (top_of_stack > bottom_of_stack);
  delete top_of_stack;
}



#endif
