// -*- C++ -*-
// varindex.c
// reads an index file, writes a letter tree
// Sven Hartrumpf (C) 1994


#include <fstream.h>
#include <std.h>

#include "types.h"
#include "vindex.h"


ULONG
count_entries(const UBYTE *name, const UBYTE * id)
{
  ULONG n = 0;
  UWORD l;
  ifstream in((const char*) name);
  if (in)
  {
    if (check_identifier(in, id))
    {
      do
      {
        l = in.get() + 3;
        //cout << "count entry, bytes: " << l - 3 << '\n'; 
        in.ignore(l);
        n++;
      }
      while (!in.eof());
      n--;
    }
  }
  return n;
}


int
main(int argc, char ** argv)
{
  if (argc < 2)
    cerr << "usage: " << argv[0] << " name_1 ... name_n\n\n";
  else
  {
    const UBYTE * name;
    ULONG n;
    UWORD i, j;
    Vindex<Index_word> * v;
    for (i = 1; i < argc; i++)
    {
      name = (const UBYTE *) argv[i];
      j = strlen((const char*) name);
      if (j < 5 ||
          (strcmp((const char*) name + j - 4, ".for") &&
           strcmp((const char*) name + j - 4, ".for")))
      {
        cerr << "Error: the input file name must end in .for or .sem\n";
      }
      else
      {
        n = count_entries(name, 0);
        if (n)
        {
          cerr << "sorting " << n << " entries in the IBL index file " << name
            << '\n';
          v = new Vindex<Index_word> (name, n, 0, 0);
          //cout << "unsorted index file\n";
          //v->print();
          Comparator<Index_word> comp;
          comp.compare = index_word_compare;
          v->sort(comp);
          v->write_ltree();
          delete v;
        }
        else
        {
          cerr << "Error: no entries in the file " << name << '\n';
          exit(1);
        }
      }
    }
  }
}
