// -*- C++ -*-
// vindex.c
// class for indexing records with variable length
// Sven Hartrumpf (C) 1994


#include "vindex.h"

#include <fstream.h>
#include <iomanip.h>
#include <std.h>


Index_word::Index_word()
{
  d = 0;
}


Index_word::~Index_word()
{
  delete d;
}


ULONG Index_word::id() const
{
  return ((((ULONG) d[1] << 8) + (ULONG) d[2]) << 8) + d[3];
}


UWORD Index_word::length() const
{
  return d[0];
}


void Index_word::print() const
{
  const UBYTE * s;
  UWORD l = *d;
  s = d + 4;
  cout << setw(l) << s;
}


void Index_word::read(ifstream & in)
{
  UBYTE * s;
  UWORD l;
  l = in.get();
  d = s = new UBYTE [l + 4];
  *s++ = l;
  l += 3;
  do
    *s++ = in.get();
  while (--l);
}


const UBYTE * Index_word::string() const
{
  return d + 4;
}


void Index_word::write(ofstream & out) const
{
  UBYTE l = (*d) + 4;
  const UBYTE * s = d;
  do
    out.put(*s++);
  while (--l);
}


BYTE index_word_compare(const Index_word & a,const Index_word & b)
{
  const UBYTE * p1 = a.d + 4;
  const UBYTE * p2 = b.d + 4;
  UWORD l1 = a.d[0];
  UWORD l2 = b.d[0];
  UWORD l = min(l1,l2);
  //cout << " index_word_compare l1: " << l1 << "  l2: " << l2 << "  ";
  //cout << "a: ";
  //a.print();
  //cout << " b:";
  //b.print();
  //cout << '\n';
  while (l)
  {
    if (*p1 < *p2)  
      return -1;
    else if (*p1 > * p2)
      return 1;
    l--;
    p1++;
    p2++;
  }
  if (l1 < l2)
    return -1;
  else if (l1 > l2)
    return 1;
  return 0;
}


BYTE check_identifier(ifstream & in, const UBYTE * id)
{
  if (id)
  {
    UWORD l = strlen((const char*) id);
    UBYTE s[l + 1];
    in.get(s, l + 1, 0);
    return (strcmp((const char*) s, (const char*) id) == 0);
  }
  return TRUE;
}
