// -*- C++ -*-
// vindex.h
// class for indexing records with variable length
// Sven Hartrumpf (C) 1994


#ifndef VINDEX_H_

#define VINDEX_H_


#include "types.h"
#include "sort.h"


#include <fstream.h>


class Index_word
{
  friend BYTE index_word_compare(const Index_word & a, const Index_word & b);
private:  
  UBYTE * d;  
public:
  Index_word();
  ~Index_word();
  ULONG id() const;
  UWORD length() const;
  void print() const;
  void read(ifstream & in);
  const UBYTE * string() const;
  void write(ofstream & out) const;
};


template<class T> class Vindex
{
private:
  ULONG num_entries;
  T * entry;
  UBYTE * name_;
  UBYTE * index_file_identifier;
  UBYTE * sorted_index_file_identifier;
public:
  Vindex(const UBYTE * name, const UBYTE * a, const UBYTE * b);
  Vindex(const UBYTE * name, ULONG size, const UBYTE * a, const UBYTE * b);
  ~Vindex();
  ULONG card() const;
  const UBYTE * name() const;
  void print() const;
  void sort(Comparator<T> & comp);
  UBYTE write(const UBYTE * name) const;
  UBYTE write_ltree() const;
  void write_ltree_aux(UWORD position, ULONG left, ULONG right, UWORD level)
                       const;
  void write_branch(UWORD position, ULONG left, ULONG right, UWORD l,
                    UWORD level) const;
};


BYTE check_identifier(ifstream & in, const UBYTE * id);


template<class T>
Vindex<T>::Vindex(const UBYTE * name, const UBYTE * a, const UBYTE * b)
{
  ifstream in((const char *) name);
  name_ = new UBYTE[strlen((const char*) name) + 1];
  strcpy((char*) name_, (const char*) name);
  if (in)
  {
    if (a)
    {
      index_file_identifier = new UBYTE[strlen((const char*) a) + 1];
      strcpy((char *) index_file_identifier, (const char*) a);
    }
    if (b)
    {
      sorted_index_file_identifier = new UBYTE[strlen((const char *) b) + 1];
      strcpy((char*) sorted_index_file_identifier, (const char*) b);
    }
    if (check_identifier(in, sorted_index_file_identifier))
    {
      in >> num_entries;
      in.get();
      cout << "read sorted index file entries " << num_entries;
      entry = new T [num_entries];
      if (entry)
      {
        for (ULONG i = 0; i < num_entries; i++)
          entry[i].read(in);
      }
    }
  }
}


template<class T>
Vindex<T>::Vindex(const UBYTE * name, ULONG size, const UBYTE * a,
                  const UBYTE * b)
{
  ifstream in((const char *) name);
  name_ = new UBYTE[strlen((const char *) name) + 1];
  strcpy((char*) name_, (const char*) name);
  if (in)
  {
    if (a)
    {
      index_file_identifier = new UBYTE[strlen((const char*) a) + 1];
      strcpy((char*) index_file_identifier, (const char*) a);
    }
    if (b)
    {
      sorted_index_file_identifier = new UBYTE[strlen((const char*) b) + 1];
      strcpy((char*) sorted_index_file_identifier, (const char*) b);
    }
    if (check_identifier(in, index_file_identifier)) 
    {
      num_entries = size;
      entry = new T [num_entries];
      if (entry)
      {
        for (ULONG i = 0; i < num_entries; i++)
          entry[i].read(in);
      }
    }
  }
}


template<class T>
Vindex<T>::~Vindex()
{
  delete name_;
  delete [] entry;
  if (index_file_identifier)
    delete index_file_identifier;
  if (sorted_index_file_identifier)
    delete sorted_index_file_identifier;
}


template<class T> const UBYTE *
Vindex<T>::name() const
{
  return name_;
}


template<class T> void
Vindex<T>::print() const
{
  for (ULONG i = 0; i < num_entries; i++)
  {
    entry[i].print();
    cout << '\n';
  }
}


template<class T> void
Vindex<T>::sort(Comparator<T> & comp)
{
  quicksort(entry,num_entries,comp);
}


template<class T> UBYTE
Vindex<T>::write(const UBYTE * name) const
{
  ofstream out((const char *) name);
  if (out)
  {
    out << sorted_index_file_identifier << num_entries << '\n'; 
    for (ULONG i = 0; i < num_entries; i++)
      entry[i].write(out);
  }
  return (!out.bad());
}


#endif
