% isobi_bp.pl
% All ISO built-in predicates.
% (See chapter 8 and 9 of the draft ISO standard for Prolog, March 1993)
% This file contains some implementations for BinProlog 2.20.
% Sven Hartrumpf (C) 1994


% standard operators

:- op(50,xfx,:).


% term unification


% =/2
% '='(?term,?term)


% unify_with_occurs_check/2
% unify_with_occurs_check(?term,?term)

%% missing


% \=/2
% '\='(@term,@term)

%% missing


% type testing


% var/1
% var(@term)


% atom/1
% atom(@term)


% integer/1
% integer(@term)


% real/1
% real(@term)

% missing


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% atomic/1
% atomic(@term)


% compound/1
% compound(@term)


% nonvar/1
% nonvar(@term)


% number/1
% number(@term)


% term comparison


% ==/2
% '=='(@term,@term)


% \==/2
% '\=='(@term,@term)


% @</2
% '@<'(@term,@term)


% @=</2
% '@=<'(@term,@term)


% @>/2
% '@>'(@term,@term)


% @>=/2
% '@>='(@term,@term)


% term creation and decomposition


% functor/3
% functor(-nonvar,+constant,+integer)
% functor(@nonvar,?constant,?integer)


% arg/3
% arg(+integer,+compound_term,?term)


% =../2
% '=..'(+nonvar,?list)
% '=..'(-nonvar,+list)


% copy_term/2
% copy_term(?term,?term)


% is/2
% 'is'(?nonvar,+nonvar)


% arithmetic comparison


% =:=/2
% '=:='(+nonvar,+nonvar)


% =\=/2
% '=\='(+nonvar,+nonvar)


% </2
% '<'(+nonvar,+nonvar)


% =</2
% '=<'(+nonvar,+nonvar)


% >/2
% '>'(+nonvar,+nonvar)


% >=/2
% '>='(+nonvar,+nonvar)


% clause retrieval and information


% clause/2
% clause(+head,?body)


% current_predicate/1
% current_predicate(?predicate_indicator)


% clause creation and destruction


% asserta/1
% asserta(@clause)


% assertz/1
% assertz(@clause)


% retract/1
% retract(+clause)


% abolish/1
% abolish(@predicate_indicator)

abolish(F/N) :-
%  functor(P,F,N),    really needed?
  functor(C,F,N),
  retractall(C),
  fail.

abolish(F/N):-
  functor(P,F,N),
  set(P,'$first',0),
  set(P,'$last',0).


% all solutions


% findall/3
% findall(@term,@callable_term,?list)


% bagof/3
% bagof(@term,+callable_term,?list)


% setof/3
% setof(@term,+callable_term,?list)


% stream selection and control


% current_input/1
% current_input(?stream)


% current_output/1
% current_output(?stream)


% set_input/1
% set_input(@stream_or_alias)


% set_output/1
% set_output(@stream_or_alias)

% open/3
% open(@source_sink,@io_mode,-stream)

open(N, read, S) :-
  seeing(Old),
  see(N),
  seeing(S),
  see(Old).


open(N, write, S) :-
  telling(Old),
  tell(N),
  telling(S),
  tell(Old).


% open/4
% open(@source_sink,@io_mode,-stream,@io_options)


% close/1
% close(@stream_or_alias)


% close/2
% close(@stream_or_alias,@close_options)

close(N, read) :-
  see(N),
  seen.

close(N, write) :-
  tell(N),
  told.


% flush_output/0
% flush_output


% flush_output/1
% flush_output(@stream_or_alias)


% stream_property/2
% stream_property(?stream,?stream_property)


% at_end_of_stream/0
% at_end_of_stream


% at_end_of_stream/1
% at_end_of_stream(@stream_or_alias)


% set_stream_position/2
% set_stream_position(@stream_or_alias,@stream_position)


% character input/output


% get_char/1
% get_char(?character)


% get_char/2
% get_char(@stream_or_alias,?character)


% put_char/1
% put_char(@character)


% put_char/2
% put_char(@stream_or_alias,@character)


% nl/0
% nl


% nl/1
% nl(@stream_or_alias)

nl(S) :-
  telling(Old),
  tell(S),
  nl,
  tell(Old).


% character code input/output


% get_code/1
% get_code(?character_code)


% get_code/2
% get_code(@stream_or_alias,?character_code)

get_code(S, Code) :-
  seeing(Old),
  see(S),
  get0(Code),
  see(Old).


% put_code/1
% put_code(@character_code)


% put_code/2
% put_code(@stream_or_alias,@code)

put_code(Out, Code) :-
  telling(Old),
  tell(Out),
  put(Code),
  tell(Old).


% term input/output


% read_term/2
% read_term(?term,+read_options_list)


% read_term/3
% read_term(@stream_or_alias,?term,+read_options_list)

read_term(S, Term, [variable_names(Vars)]) :-
  seeing(Old),
  see(S),
  read_term(Term, Vars),
  see(Old).

read_term(S, Term, []) :-
  seeing(Old),
  see(S),
  read(Term),
  see(Old).


% read/1
% read(?term)


% read/2
% read(@stream_or_alias,?term)


% write_term/2
% write_term(@term,@write_options_list)


% write_term/3
% write_term(@stream_or_alias,@term,@write_options_list)


% write/1
% write(@term)


% write/2
% write(@stream_or_alias,@term)

write(Out, T) :-
  telling(Old),
  tell(Out),
  write(T),
  tell(Old).


% writeq/1
% writeq(@term)


% writeq/2
% writeq(@stream_or_alias,@term)


% write_canonical/1
% write_canonical(@term)


% write_canonical/2
% write_canonical(@stream_or_alias,@term)


% op/3
% op(@integer,@operator_specifier,@atom_or_atom_list)


% current_op/3
% op(?integer,?operator_specifier,?atom)


% char_conversion/2
% char_conversion(@character,@character)


% current_char_conversion/2
% current_char_conversion(?character,?character)


% logic and control


% fail_if/1
% fail_if(@callable_term)


% once/1
% once(+callable_term)


% repeat/0
% repeat


% constant processing


% atom_length/2
% atom_length(+atom,?integer)

atom_length(A, L) :-
  atom_codes(A, Codes),
  length(Codes, L).


% atom_concat/3
% atom_concat(?atom,?atom,+atom)
% atom_concat(+atom,+atom,-atom)

atom_concat(A,B,C) :-
  name(A,A_string),
  name(B,B_string),
  append(A_string,B_string,C_string),
  name(C,C_string).


% sub_atom/4
% sub_atom(+atom,?integer,?integer,?atom)


% atom_chars/2
% atom_chars(+atom,+list)
% atom_chars(+atom,-list)
% atom_chars(-atom,+list)


% atom_codes/2
% atom_codes(+atom,+list)
% atom_codes(+atom,-list)
% atom_codes(-atom,+list)

atom_codes(A,L) :-
  name(A,L).


% char_code/2
% char_code(+character,+character_code)
% char_code(+character,-character_code)
% char_code(-character,+character_code)


% number_chars/2
% number_chars(+number,+list)
% number_chars(+number,-list)
% number_chars(-number,+list)


% number_codes/2
% number_codes(+number,+list)
% number_codes(+number,-list)
% number_codes(-number,+list)

number_codes(N, L) :-
  name(N, L).


% implementation defined hooks


% set_prolog_flag/2
% set_prolog_flag(@flag,@term)


% current_prolog_flag/2
% current_prolog_flag(?flag,?term)


% halt/0
% halt


% halt/1
% halt(@integer)



% mathematic predicates
% min(A,B,M)
% M is the minimum of A and B.

min(A,B,M) :-
  A =< B,
  !,
  M = A.

min(_,B,B).
