% lists.pl
% predicates for lists
% Sven Hartrumpf (C) 1994


% append/3
% append(A,B,C) is true iff
%   C is the concatenation of the lists A and B.
% append(+list,+list,+list)
% append(+list,+list,-list)
% append(+list,-list,+list)
% append(-list,+list,+list)
% append(-list,-list,+list)

/*append([],L,L).

append([X|A],B,[X|C]) :-
  append(A,B,C).*/


% is_list/1
% is_list(L) is true iff
%   L is a list.
% is_list(@term)

is_list(L) :-
  nonvar(L),
  (
    L = []
  ;
    L = [_|_]
  ).


% last_element/2
% last_element(L,E) is true iff
%   E is the last element in the list L.
% last_element(+list,?term)

last_element([E],E).

last_element([_|L],E) :-
  last_element(L,E).


% list_intersection/3
% list_intersection(A,B,C) is true iff
%   C is the list A without the elements that are not in the list B.
% list_intersection(@list,@list,?list)

list_intersection([],_,[]).

list_intersection([H|T],L,[H|Result]) :-
  member(H,L),
  !,
  list_intersection(T,L,Result).

list_intersection([_|T],L,Result) :-
  list_intersection(T,L,Result).


% list_subtraction/3
% list_subtraction(A,B,C) is true iff
%   C is the list A without the elements that are in the list B.
% list_subtraction(@list,@list,?list)

list_subtraction([],_,[]).

list_subtraction([H|T],L,[H|Result]) :-
  nonmember(H,L),
  !,
  list_subtraction(T,L,Result).

list_subtraction([_|T],L,Result) :-
  list_subtraction(T,L,Result).


% map_list/3
% map_list(P,I,O) is true iff
%   - O is a list resulting from mapping the list I using the predicate (and
%     leading arguments) in P, and
%   - P is a lambda expression: the first lambda operator supplies the input
%     argument, the second, the output argument.
% map_list(+term,+list,?list)

map_list(I^O^Goal,Is,Os):-
  findall(O,(member(I,Is),Goal),Os).


% member/2
% member(E,L) is true iff
%   E is an element of the list L.
% Comments: Succeeds for every element E in L once.
% member(?term,+list)

/*member(E,[E|_]).

member(E,[_|L]) :-
  member(E,L).*/


% memberv/2
% memberv(E,L) is true iff
%   E is an element of the list L.
% Comments: L may contain variables.
% memberv(?term,+list)

memberv(A,[B|_]) :-
  A == B.

memberv(A, [_|L]) :-
  memberv(A,L).


% member_check/2
% member_check(E,L) is true iff
%   E is an element of the list L.
% member_check(?term,+list)

member_check(E,[H|_]) :-
  E == H,
  !.

member_check(E,[_|L]) :-
  member_check(E,L).


% nonmember/2
% nonmember(E,L) is true iff
%   E is not an element of the list L.
% Comments: L may contain variables, but may not be an open list.
% nonmember(?term,+list)

nonmember(_,[]).

nonmember(A,[B|L]) :-
  A \== B,
  nonmember(A,L).


% nonmemberv/2
% nonmemberv(E,L) is true iff
%   E is not an element of the list L.
% Comments: L may be an open list.
% nonmemberv(?term,+list)

nonmemberv(_,L) :-
  var(L),
  !.

nonmemberv(_,[]).

nonmemberv(A,[B|L]) :-
  A \== B,
  nonmemberv(A,L).


% nth0/3
% nth0(N,L,E) is true iff
%   E is the N-th element in the list L. (The first element has the number 0.)
% nth0(+integer,+list,?term)
% nth0(-integer,+list,+term)

nth0(N,L,E) :-
  nonvar(N),
  !,
  nth0_1(N,L,E).

nth0(N,L,E) :-
  nth0_2(N,L,E).


nth0_1(0,[E|_],E) :-
  !.

nth0_1(N,[_|L],E) :-
  M is N - 1,
  nth0_1(M,L,E).


nth0_2(0,[E|_],E) :-
  !.

nth0_2(N,[_|L],E) :-
  nth0_2(M,L,E),
  N is M + 1.


% nth1/3
% nth1(N,L,E) is true iff
%   E is the N-th element in the list L. (The first element has the number 1.)
% nth1(+integer,+list,?term)
% nth1(-integer,+list,+term)

nth1(N,L,E) :-
  nonvar(N),
  !,
  M is N - 1,
  nth0(M,L,E).

nth1(N,L,E) :-
  nth0(M,L,E),
  N is M + 1.


% nth_occurrence/4
% nth_occurrence(O,L,E,N) is true iff
%   - E is the N-th element in the list L. (The first element has the number
%     1.), and
%   - at the position N is the O-th occurence of the element E in the list L.
% nth_occurrence(+integer,+list,+term,?integer)

nth_occurrence(O,L,E,N) :-
  nth_occurrence2(O,L,E,1,N).

nth_occurrence2(1,[E|_],E,N,N) :-
  !.

nth_occurrence2(O,[E|L],E,N,P) :-
  !,
  N1 is N + 1,
  O1 is O - 1,
  nth_occurrence2(O1,L,E,N1,P).

nth_occurrence2(O,[_|L],E,N,P) :-
  N1 is N + 1,
  nth_occurrence2(O,L,E,N1,P).


% process_list/2
% process_list(G,L) is true iff
%   - the goal G has been executed for all elements in L, and
%   - G is a lambda expression supplying an input argument from the list L. 
% process_list(+term,+list)

process_list(I^Goal,Is) :-
  member(I,Is),
  Goal,
  fail.

process_list(_,_).


% remove_front_doubles/2
% remove_front_doubles(L,N) is true iff
%   N is the list L without doubles. Only the right most elements are included
%   in N, if there are doubles.
% remove_front_doubles(+list,?list)

remove_front_doubles([],[]).

remove_front_doubles([H|T],List) :-
  remove_front_doubles(T,List1),
  (
    member_check(H,List1)
  ->
    List = List1
  ;
    List = [H|List1]
  )
.


% reverse_list/2
% reverse_list(A,B) is true iff
%   B is the reverse of the list A.
% reverse_list(+list,?list)

reverse_list(L,R) :-
  reverse_list(L,[],R).

reverse_list([],R,R).

reverse_list([H|T],L,R) :-
  reverse_list(T,[H|L],R).


/* more tricky but only half as fast as the easy one

reverse_list([],[]) :-
  !.

reverse_list([E],[E]) :-
  !.

reverse_list([E1,E2],[E2,E1]) :-
  !.

reverse_list(A,B) :-
  reverse_list2(A,B,[]).

% reverse_list2(L,R,Hole)
% The list L is reversed to the list R. Hole is a variable that describes the
% hole at the end of the current result.
% For better performance, two elements are processed per step.
% This predicate is tail recursive.

reverse_list2([E],[E|Hole],Hole) :-
  !.

reverse_list2([E1,E2],[E2,E1|Hole],Hole) :-
  !.

reverse_list2([E1,E2|T],Result,Previous_hole) :-
  reverse_list2(T,Result,[E2,E1|Previous_hole]).
*/
